@@ -14,6 +14,8 @@ import time
 import itertools
 import copy
 from functools import partial
+import multiprocessing
+import math
 
 logger = logging.getLogger(__name__)
 
@@ -43,6 +45,9 @@ def reconstruct_poses_3d(
     pose_3d_graph_initial_edge_threshold=poseconnect.defaults.RECONSTRUCTION_POSE_3D_GRAPH_INITIAL_EDGE_THRESHOLD,
     pose_3d_graph_max_dispersion=poseconnect.defaults.RECONSTRUCTION_POSE_3D_GRAPH_MAX_DISPERSION,
     include_track_labels=poseconnect.defaults.RECONSTRUCTION_INCLUDE_TRACK_LABELS,
+    parallel=poseconnect.defaults.RECONSTRUCTION_PARALLEL,
+    num_parallel_processes=poseconnect.defaults.RECONSTRUCTION_NUM_PARALLEL_PROCESSES,
+    num_chunks=poseconnect.defaults.RECONSTRUCTION_NUM_CHUNKS,
     progress_bar=poseconnect.defaults.PROGRESS_BAR,
     notebook=poseconnect.defaults.NOTEBOOK
 ):
@@ -78,6 +83,137 @@ def reconstruct_poses_3d(
             room_y_limits=room_y_limits,
             pose_model_name=pose_model_name
         )
+    num_frames = len(poses_2d['timestamp'].unique())
+    logger.info('Reconstructing 3D poses from {} 2D poses across {} frames ({} to {})'.format(
+        len(poses_2d),
+        num_frames,
+        poses_2d['timestamp'].min().isoformat(),
+        poses_2d['timestamp'].max().isoformat()
+    ))
+    start_time = time.time()
+    if not parallel:
+        poses_3d = reconstruct_poses_3d_chunk(
+            poses_2d=poses_2d,
+            camera_calibrations=camera_calibrations,
+            pose_3d_limits=pose_3d_limits,
+            min_keypoint_quality=min_keypoint_quality,
+            min_num_keypoints=min_num_keypoints,
+            min_pose_quality=min_pose_quality,
+            min_pose_pair_score=min_pose_pair_score,
+            max_pose_pair_score=max_pose_pair_score,
+            pose_pair_score_distance_method=pose_pair_score_distance_method,
+            pose_pair_score_pixel_distance_scale=pose_pair_score_pixel_distance_scale,
+            pose_pair_score_summary_method=pose_pair_score_summary_method,
+            pose_3d_graph_initial_edge_threshold=pose_3d_graph_initial_edge_threshold,
+            pose_3d_graph_max_dispersion=pose_3d_graph_max_dispersion,
+            include_track_labels=include_track_labels,
+            progress_bar=progress_bar,
+            notebook=notebook
+        )
+    else:
+        poses_2d = poses_2d.sort_values('timestamp')
+        timestamps = poses_2d['timestamp'].unique()
+        num_timestamps = len(timestamps)
+        if num_parallel_processes is None:
+            num_cpus=multiprocessing.cpu_count()
+            num_processes = min(num_cpus - 1, num_timestamps)
+            logger.info('Number of parallel processes not specified. Data spans {} frames and {} CPUs detected. Launching {} processes'.format(
+                num_timestamps,
+                num_cpus,
+                num_processes
+            ))
+        if num_chunks is None:
+            num_chunks = min(num_processes, num_timestamps)
+            logger.info('Number of chunks not specified. Data spans {} frames and {} processes will be used. Breaking the data into {} chunks'.format(
+                num_timestamps,
+                num_processes,
+                num_chunks
+            ))
+        logger.info('Using {} processes to process {} frames broken into {} chunks'.format(
+            num_processes,
+            num_timestamps,
+            num_chunks
+        ))
+        timestamp_chunks = np.array_split(timestamps, num_chunks)
+        poses_2d_chunk_list = list()
+        for chunk_index, timestamp_chunk in enumerate(timestamp_chunks):
+            poses_2d_chunk = poses_2d.loc[poses_2d['timestamp'].isin(timestamp_chunk)]
+            logger.debug('Chunk {}: {} poses spanning {} timestamps from {} to {}'.format(
+                chunk_index,
+                len(poses_2d_chunk),
+                poses_2d_chunk['timestamp'].nunique(),
+                poses_2d_chunk['timestamp'].min().isoformat(),
+                poses_2d_chunk['timestamp'].max().isoformat()
+            ))
+            poses_2d_chunk_list.append(poses_2d_chunk)
+        reconstruct_poses_3d_chunk_partial = partial(
+            reconstruct_poses_3d_chunk,
+            camera_calibrations=camera_calibrations,
+            pose_3d_limits=pose_3d_limits,
+            min_keypoint_quality=min_keypoint_quality,
+            min_num_keypoints=min_num_keypoints,
+            min_pose_quality=min_pose_quality,
+            min_pose_pair_score=min_pose_pair_score,
+            max_pose_pair_score=max_pose_pair_score,
+            pose_pair_score_distance_method=pose_pair_score_distance_method,
+            pose_pair_score_pixel_distance_scale=pose_pair_score_pixel_distance_scale,
+            pose_pair_score_summary_method=pose_pair_score_summary_method,
+            pose_3d_graph_initial_edge_threshold=pose_3d_graph_initial_edge_threshold,
+            pose_3d_graph_max_dispersion=pose_3d_graph_max_dispersion,
+            include_track_labels=include_track_labels,
+            progress_bar=False,
+            notebook=notebook
+        )
+        with multiprocessing.Pool(num_processes) as p:
+            if progress_bar:
+                if notebook:
+                    poses_3d = pd.concat(tqdm.notebook.tqdm(
+                        p.imap_unordered(
+                            reconstruct_poses_3d_chunk_partial,
+                            poses_2d_chunk_list
+                        ),
+                        total=len(poses_2d_chunk_list)
+                    ))
+                else:
+                    poses_3d = pd.concat(tqdm.tqdm(
+                        p.imap_unordered(
+                            reconstruct_poses_3d_chunk_partial,
+                            poses_2d_chunk_list
+                        ),
+                        total=len(poses_2d_chunk_list)
+                    ))
+            else:
+                poses_3d = pd.concat(p.imap_unordered(
+                        reconstruct_poses_3d_chunk_partial,
+                        poses_2d_chunk_list
+                ))
+        poses_3d.sort_values('timestamp', inplace=True)
+    elapsed_time = time.time() - start_time
+    logger.info('Generated {} 3D poses in {:.1f} seconds ({:.3f} ms/frame)'.format(
+        len(poses_3d),
+        elapsed_time,
+        1000*elapsed_time/num_frames
+    ))
+    return poses_3d
+
+def reconstruct_poses_3d_chunk(
+    poses_2d,
+    camera_calibrations,
+    pose_3d_limits,
+    min_keypoint_quality=poseconnect.defaults.RECONSTRUCTION_MIN_KEYPOINT_QUALITY,
+    min_num_keypoints=poseconnect.defaults.RECONSTRUCTION_MIN_NUM_KEYPOINTS,
+    min_pose_quality=poseconnect.defaults.RECONSTRUCTION_MIN_POSE_QUALITY,
+    min_pose_pair_score=poseconnect.defaults.RECONSTRUCTION_MIN_POSE_PAIR_SCORE,
+    max_pose_pair_score=poseconnect.defaults.RECONSTRUCTION_MAX_POSE_PAIR_SCORE,
+    pose_pair_score_distance_method=poseconnect.defaults.RECONSTRUCTION_POSE_PAIR_SCORE_DISTANCE_METHOD,
+    pose_pair_score_pixel_distance_scale=poseconnect.defaults.RECONSTRUCTION_POSE_PAIR_SCORE_PIXEL_DISTANCE_SCALE,
+    pose_pair_score_summary_method=poseconnect.defaults.RECONSTRUCTION_POSE_PAIR_SCORE_SUMMARY_METHOD,
+    pose_3d_graph_initial_edge_threshold=poseconnect.defaults.RECONSTRUCTION_POSE_3D_GRAPH_INITIAL_EDGE_THRESHOLD,
+    pose_3d_graph_max_dispersion=poseconnect.defaults.RECONSTRUCTION_POSE_3D_GRAPH_MAX_DISPERSION,
+    include_track_labels=poseconnect.defaults.RECONSTRUCTION_INCLUDE_TRACK_LABELS,
+    progress_bar=poseconnect.defaults.PROGRESS_BAR,
+    notebook=poseconnect.defaults.NOTEBOOK
+):
     reconstruct_poses_3d_timestamp_partial = partial(
         reconstruct_poses_3d_timestamp,
         camera_calibrations=camera_calibrations,
@@ -94,14 +230,6 @@ def reconstruct_poses_3d(
         pose_3d_graph_max_dispersion=pose_3d_graph_max_dispersion,
         include_track_labels=include_track_labels
     )
-    num_frames = len(poses_2d['timestamp'].unique())
-    logger.info('Reconstructing 3D poses from {} 2D poses across {} frames ({} to {})'.format(
-        len(poses_2d),
-        num_frames,
-        poses_2d['timestamp'].min().isoformat(),
-        poses_2d['timestamp'].max().isoformat()
-    ))
-    start_time = time.time()
     poses_3d_list = list()
     if progress_bar:
         if notebook:
@@ -114,12 +242,6 @@ def reconstruct_poses_3d(
         for timestamp, group_df in poses_2d.groupby('timestamp'):
             poses_3d_list.append(reconstruct_poses_3d_timestamp_partial(group_df))
     poses_3d = pd.concat(poses_3d_list)
-    elapsed_time = time.time() - start_time
-    logger.info('Generated {} 3D poses in {:.1f} seconds ({:.3f} ms/frame)'.format(
-        len(poses_3d),
-        elapsed_time,
-        1000*elapsed_time/num_frames
-    ))
     return poses_3d
 
 def pose_3d_limits_by_pose_model(
